#!/usr/bin/env python

"""
Command utility to examine a SEGY file

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segydump.py,v 1.14 2005/09/15 13:45:52 schwehr Exp $

http://www.python.org/doc/lib/module-optparse.html

http://schwehr.org/software/segy-py/
"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.14 $' # See man ident
__date__ = '$Date: 2005/09/15 13:45:52 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
__credits__ = """Lots of people for helping me, especially Neal
Driscoll and Lisa Tauxe."""

import os
import sys

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


from segy import Segy
import segy

from optparse import OptionParser

# What version do we want to track?  This would be for cvs of this file
vcvs = "$Revision: 1.14 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] segyfile1 segyfile2 ... segyfileN",
                      version="%prog "+VERSION+' - '+version_cvs)

myparser.add_option('-d','--driver',dest="drv",
                    help="specify which driver to parse data file.  [default: %default]",
                    default="segy_drv_rev1")
# Match -H with grep
myparser.add_option('-H','--with-filename',action='store_true',dest='withFilename',default=False,
                    help='print the filename for each entry')
myparser.add_option('-n','--no-file-header',action="store_false",dest='fileHeader',
                    help="do not show the file header fields",default=True)
myparser.add_option('-N','--get-num-traces',action="store_true",dest='getNumTraces',
                    help='print out the number of traces in a file')
myparser.add_option('-s','--short-list',action="store_true",dest='shortList',default=False,
                    help='restrict the fields to the drivers short list')
myparser.add_option('-t','--trace-header',action="store",dest='traceHeaderNumber',
                    help='set this to view values for a trace header.  0 for all.  Negative values go from the back of the file. [default: %default]',
                    default=None, type='int')
myparser.add_option('-T','--with-trace-num',action='store_true',dest='withTraceNum',default=False,
                    help='print the trace number before each trace entry')
myparser.add_option('-w','--write-trace',action="store_true",dest='writeTrace',default=False,
                    help='use this flag to write the trace files to disk')
myparser.add_option('-v','--verbose',action="count",dest='verbosity',default=0,
                    help='how information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')



def dumpFile(filename,options,v):
    """
    Sent relevant info to stdout or out to a file for one particular segy file
    """
    if v>=TRACE: print '\n########################################'
    print 'filename="'+filename+'"'
    if v>=TRACE: print '########################################'

    s = Segy(filename=filename,drivername=options.drv,
             verbosity=options.verbosity)

    if options.getNumTraces:
        print 'numTracesInFile='+str(s.getNumberOfTraces())

    if options.fileHeader and (not options.writeTrace):
        print '\n# FILE HEADER'
        if options.withFilename:
            s.header.printBinaryHeaders(options.shortList,filename)
        else:
            s.header.printBinaryHeaders(shortList=options.shortList)

    if options.traceHeaderNumber and 0>options.traceHeaderNumber:
        # go from the end if negative
        # FIX: make sure this works right!
        options.traceHeaderNumber = (s.getNumberOfTraces()+1)+options.traceHeaderNumber

    if options.writeTrace:
            
        if not options.traceHeaderNumber: traceHeaderNumber = 1
        if 0 == options.traceHeaderNumber:
            for i in range(1,s.getNumberOfTraces()+1):
                tStr = "%05d" % i
                outName = filename+"-"+tStr+'.dat'
                if v>=TRACE: print outName
                s.writeTraceToFile(filename+str(i)+'.dat',traceNumber=i)
              
        else:
            i = options.traceHeaderNumber
            tStr = "%05d" % i
            outName = filename+"-"+tStr+'.dat'
            print outName
            s.writeTraceToFile(outName,traceNumber=i)

        sys.exit("skip printing to stdio")

    if None != options.traceHeaderNumber:
        if options.withFilename: f = filename
        else: f = None
        if 0 == options.traceHeaderNumber:
            if v>=VERBOSE: print 'Dumping all'
            for i in range(1,s.getNumberOfTraces()+1):
                if options.withTraceNum: t = i
                else: t = None
                print '\n# TRACE HEADER',i
                if options.shortList:
                    s.getTraceHdr(options.traceHeaderNumber).printHeader(shortList=options.shortList, filename=f, traceNum=t)
                else:
                    s.getTraceHdr(options.traceHeaderNumber).printHeader(filename=f,traceNum=t)
        else:
            if options.traceHeaderNumber: t = options.traceHeaderNumber
            else: t = None
            if v>=TERSE: print '\n# TRACE HEADER',options.traceHeaderNumber
            if options.shortList:
                s.getTraceHdr(options.traceHeaderNumber).printHeader(shortList=options.shortList,filename=f, traceNum=t)
            else:
                s.getTraceHdr(options.traceHeaderNumber).printHeader(filename=f,traceNum=t)


if __name__ == '__main__':
    (options,args) = myparser.parse_args()
    v = options.verbosity
    if v >= VERBOSE:   print 'Option values: ', options
    if v >= BOMBASTIC: print 'Arg    values: ', args
    # Args contains other files to read
    for filename in args:
        dumpFile(filename,options,v)
