#!/usr/bin/env python

"""

Slice out traces from a segy file

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segyslice.py,v 1.2 2005/09/15 15:15:41 schwehr Exp $

http://www.python.org/doc/lib/module-optparse.html

http://schwehr.org/software/segy-py/
"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.2 $' # See man ident
__date__ = '$Date: 2005/09/15 15:15:41 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
#__credits__ = """"""

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


import os
import sys

from segy import Segy
import segy

from optparse import OptionParser

vcvs = "$Revision: 1.2 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] segyfile1 segyfile2 ... segyfileN",
                      version="%prog "+VERSION+' - '+version_cvs)

myparser.add_option('-d','--driver',dest="drv",
                    help="specify which driver to parse data file.  [default: %default]",
                    default="segy_drv_rev1")

myparser.add_option('-o','--out',dest="outfile",
                    help="Write segy to this file.  [default: %default]",
                    default="out.sgy")

myparser.add_option('-i','--in',dest="infile",
                    help="Read segy to this file.  [default: %default]",
                    default="in.sgy")

myparser.add_option('-m','--min',dest='min',default=None, type='int',
                    help='Minimum trace to start writing [default: %default]')
myparser.add_option('-M','--max',dest='max',default=None, type='int',
                    help='Maximum trace to end writing on [default: %default]')

# Put a whole in the data
myparser.add_option('-e','--min-except',dest='minExcept',default=None, type='int',
                    help='Minimum trace to start skipping [default: %default]')
myparser.add_option('-E','--max-except',dest='maxExcept',default=None, type='int',
                    help='Maximum trace to end skipping [default: %default]')
# FIX: make sure that if -e then also -E



myparser.add_option('-v','--verbose',action="count",dest='verbosity',default=0,
                    help='how information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')

######################################################################



if __name__ == '__main__':
    (options,args) = myparser.parse_args()
    
    if options.verbosity >= VERBOSE:   print 'Option values: ', options
    if options.verbosity >= BOMBASTIC: print 'Arg    values: ', args

    if ((options.minExcept != None and options.maxExcept == None) or
        (options.minExcept == None and options.maxExcept != None)):
        sys.exit("ERROR: must specify both -e and -E")

    if options.minExcept != None and options.minExcept > options.maxExcept:
        sys.exit("ERROR: min except must be <= max except")
        

    s = Segy(options.infile, options.drv, options.verbosity)

    
    numTraces = s.getNumberOfTraces()
    #print numTraces

    if options.min == None: options.min = 1 # FIX: start counting from 1?????
    if options.max == None: options.min = s.getNumberOfTraces() # FIX: off by one?
        
    traces= []
    if options.minExcept != None:
        # poke a hole
        traces = range(options.min, options.minExcept)
        traces += range(options.maxExcept+1,options.max)
    else:
        traces = range(options.min,options.max)
    #print traces

    out = open(options.outfile,'w')

    # Duplicate the text and binary headers
    hdr = s.getHeader().getEntireRawHdr()
    #print len(hdr)
    out.write(hdr)

    for traceNum in traces:
        # NOP
        #print traceNum
        if options.verbosity >= BOMBASTIC: print traceNum
        elif options.verbosity >= TRACE:
            if (traceNum%100)==0:
                print traceNum
            
        d = s.getRawTrace(traceNum)
        #print 'tracesize = ',len(d)
        out.write(d)
        
